#include <cstdio>
#include <map>
#include <vector>
#include <cmath>
#include <algorithm>
#include <iostream>
#include <set>
#include <cstring>
#include <queue>
#include <ctime>
#include <fstream>
#include <cassert>

#define mp make_pair
#define pii pair<int, int>
#define si short int

using namespace std;

const int MAX_LABELS = 20;

int MAXSIZE = 30;

struct rgb {
	rgb (int _r = 0, int _g = 0, int _b = 0) : r(_r), g(_g), b(_b) {}
	
	int r, g, b;
	
	bool equals (rgb other) {
		
		return other.r == r && other.g == g && other.b == b;
	}
	
	void print () {
		cout << "rgb: " << r << ", " << g << ", " << b << "\n";
	}
};

struct oper {
	int x, y;
	int x2, y2;
	int h, w;
	
	oper () {};
	oper (int _x = 0, int _y = 0, int _x2 = 0, int _y2 = 0, int _h = 0, int _w = 0) : x(_x), y(_y), x2(_x2), y2(_y2), h(_h), w(_w) {}
};

struct rect {
	int x, y;
	int h, w;
	long long gainval;
	
	rect () {}
	rect (int _x, int _y, int _h, int _w, long long _gain) : x(_x), y(_y), h(_h), w(_w), gainval(_gain) {}
};

bool cmpGainRect (const rect &p1, const rect &p2) {
	
	return p1.gainval > p2.gainval;
}

int H, W;
int maxOperations;

rgb startImage[256][256];
rgb targetImage[256][256];
rgb need_image[256][256];
rgb current_image[256][256];
rgb tempImage[256][256];

vector <oper> ans_oper;
vector <rect> opertry;

int labels_need[256][256], labels_current[256][256];

int msize[256][256];

bool covered[256][256];
int godown[256][256];
int goright[256][256];

int labelBestSize[MAX_LABELS];
int labelBestSizeRect[MAX_LABELS][2];

void readImage(rgb img[256][256]) {

	for (int r = 0; r < 3; ++r) {
		for (int i = 0; i < H; ++i)
			for (int j = 0; j < W; ++j)
			{
				if (r == 0) 
					scanf ("%d", &img[i][j].r);
				else 
				if (r == 1)
					scanf ("%d", &img[i][j].g);
				else
					scanf ("%d", &img[i][j].b);
			}
	}
}

void input() {

	scanf ("%d%d%d", &H, &W, &maxOperations);
	
	readImage (startImage);
	readImage (targetImage);
}

void update_start_image(int x, int y, int x2, int y2, int h, int w) {
	
	for (int i = 0; i < h; ++i)
		for (int j = 0; j < w; ++j) {
			startImage[x+i][y+j].r += startImage[x2+i][y2+j].r;
			startImage[x+i][y+j].g += startImage[x2+i][y2+j].g;
			startImage[x+i][y+j].b += startImage[x2+i][y2+j].b;
			
			startImage[x+i][y+j].r /= 2;
			startImage[x+i][y+j].g /= 2;
			startImage[x+i][y+j].b /= 2;
		}
}

bool overlapp(int x, int y, int x2, int y2, int n, int m) {
	if (x + n <= x2) return false;
	if (x2 + n <= x) return false;
	if (y + m <= y2) return false;
	if (y2 + m <= y) return false;
	
	return true;
}

void calculate_godownright(int table[256][256]) {
	for (int j = W - 1; j >= 0; --j)
		for (int i = H - 1; i >= 0; --i) {
		
			if (j + 1 < W && table[i][j] == table[i][j+1])
				goright[i][j] = goright[i][j+1] + 1;
			else 
				goright[i][j] = 1;
				
			if (i + 1 < H && table[i][j] == table[i+1][j])
				godown[i][j] = godown[i+1][j] + 1;
			else
				godown[i][j] = 1;
		}
}

inline int getGrayscaleK(const int &r, const int &g, const int &b, const int &k) {

    //convert to grayscale
    double res = 0.2126*r + 0.7152*g + 0.0722*b;
    if (res <= 0) res = 0;
    if (res > 255) res = 255;

    //compute bin into nval
    int nval = (int)res; nval *= k;
    
	nval /= 255;
	
	if (nval >= k)
		nval = k - 1;

    return nval;
}

void fixRgb (rgb &other) {

	other.r = max(other.r, 0);
	other.g = max(other.g, 0);
	other.b = max(other.b, 0);
	
	other.r = min(other.r, 255);
	other.g = min(other.g, 255);
	other.b = min(other.b, 255);
}

rgb compute_need_pixel(const rgb &start, const rgb &target) {
	rgb result(2.0 * target.r - start.r, 2.0 * target.g - start.g, 2.0 * target.b - start.b);
	fixRgb(result);
	return result;
}

void gaussianSmooth(rgb img[256][256]) {
			
    //smoothing with Gaussian filter
    const int matr[5][5] = {{2,4,5,4,2},{4,9,12,9,4},{5,12,15,12,5},{4,9,12,9,4},{2,4,5,4,2}};
    int totalM = 0;


    for (int x = 0; x < H; ++x) {
        for (int y = 0; y < W; ++y) {
            double r = 0, g = 0, b = 0;
            totalM = 0;

            for (int i = -2; i <= 2; ++i)
                for (int j = -2; j <= 2; ++j) {

                    if (x + i >= 0 && x + i < H)
                        if (y + j >= 0 && y + j < W) {

                            r += (img[x+i][y+j].r * matr[i+2][j+2]);
                            g += (img[x+i][y+j].g * matr[i+2][j+2]);
                            b += (img[x+i][y+j].b * matr[i+2][j+2]);
                            totalM += matr[i+2][j+2];
                        }
                }

            if (totalM != 0) {
                r /= totalM;
                g /= totalM;
                b /= totalM;
                tempImage[x][y].r = r; tempImage[x][y].g = g; tempImage[x][y].b = b;
            }
        }
    }
    
    for (int i = 0; i < H; ++i)
		for (int j = 0; j < W; ++j) {
			img[i][j].r = tempImage[i][j].r;
			img[i][j].g = tempImage[i][j].g;
			img[i][j].b = tempImage[i][j].b;
		}
}

void posterizeImage(rgb img[256][256], int labels[256][256], int clusters[MAX_LABELS][3], int count[MAX_LABELS], int k) {
	
	for (int i = 0; i < MAX_LABELS; ++i) {
		clusters[i][0] = clusters[i][1] = clusters[i][2] = count[i] = 0;
	}
	
	int r, g, b;
	
	for (int i = 0; i < H; ++i)
		for (int j = 0; j < W; ++j)
         {
			r = img[i][j].r; g = img[i][j].g; b = img[i][j].b;
			int label = getGrayscaleK(r, g, b, k);
			
			labels[i][j] = label;
			
			assert (label >= 0);
			assert (label < MAX_LABELS);
			assert (r >= 0 && r <= 255);
			assert (g >= 0 && g <= 255);
			assert (b >= 0 && b <= 255);
			
			clusters[label][0] += r;
			clusters[label][1] += g;
			clusters[label][2] += b;
			count[label]++;
			
			assert (clusters[label][2] / count[label] <= 255);
			assert (clusters[label][0] / count[label] <= 255);
			assert (clusters[label][1] / count[label] <= 255);
	}
	
	for (int i = 0; i < k; ++i)
		if (count[i] > 0) {
			
			clusters[i][0] /= count[i];
			clusters[i][1] /= count[i];
			clusters[i][2] /= count[i];
		}
		else {
			clusters[i][0] = clusters[i][1] = clusters[i][2] = 0;
		}
}

inline int dist(const rgb &x,const rgb &y) {

	int a = x.r - y.r;
	int b = x.g - y.g;
	int c = x.b - y.b;
	
	return a*a + b*b + c*c;
}

double calculate_gain(const rgb &original, const rgb &target, const rgb &afterfix) {
	
	int afterD = dist(afterfix, target);
	int initialD = dist(original, target);
	
	return sqrt(initialD) - sqrt(afterD);
}


void dump_image (rgb img[256][256], string fname) {
	
	ofstream fout(fname.c_str());
	
	fout << H << " " << W << " ";
	for (int i = 0; i < H; ++i) {
		for (int j = 0; j < W; ++j)
			fout << img[i][j].r << " " << img[i][j].g << " " << img[i][j].b << " ";
	}
			
	fout.close();
}

void dump_it_now (int dumpId) {
	
	char haha[100];
	sprintf (haha, "need_label_%d.txt", dumpId);
	
	ofstream fout(haha);
	
	//fout << H << " " << W << " ";
	for (int i = 0; i < H; ++i) {
		for (int j = 0; j < W; ++j)
			fout << labels_need[i][j] << " ";
			
		fout << "\n";
	}
			
	fout.close();
}
bool check_covered(int x, int y, int h, int w) {
	
	for (int x2 = x; x2 < x + h; x2++)
		for (int y2 = y; y2 < y + w; y2++)
			if (covered[x2][y2])
				return true;
				
	return false;
}

void mark_covered(int x, int y, int h, int w) {
	
	for (int x2 = x; x2 < x + h; x2++)
		for (int y2 = y; y2 < y + w; y2++)
			covered[x2][y2] = 1;
}

double gainOper2(int x, int y, int x2, int y2, int h, int w) {
	
	double ansgain = 0;
	
	for (int i = 0; i < h; ++i)
		for (int j = 0; j < w; ++j) {
			
			rgb afterFix ((startImage[x2+i][y2+j].r + startImage[x+i][y+j].r)/2, 
						  (startImage[x2+i][y2+j].g + startImage[x+i][y+j].g)/2, 
						  (startImage[x2+i][y2+j].b + startImage[x+i][y+j].b)/2);
						  
			ansgain += calculate_gain(startImage[x+i][y+j],targetImage[x+i][y+j],afterFix);
		}
	
	return ansgain;
}

void compute_squares(int maxsize[256][256], int table[256][256], int k) {

	calculate_godownright(table);
	
	for (int i = 0; i < k; ++i)
		labelBestSize[i] = -1;
		
	for (int i = H - 1; i >= 0; --i)
		for (int j = W - 1; j >= 0; --j) {
		
			maxsize[i][j] = 1;
			
			if (table[i][j] < 0 || table[i][j] > k)
				continue;
				
			int sevalue = 0;
			if (i + 1 < H && j + 1 < W && table[i][j] == table[i+1][j+1])
				sevalue = maxsize[i+1][j+1];
			sevalue++;
			
			sevalue = min (godown[i][j], min(sevalue, goright[i][j]));
			maxsize[i][j] = sevalue;
			
			int lbl = table[i][j];
			int sz = maxsize[i][j];
			
			assert (sz > 0);
			
			sz = min (sz, MAXSIZE);
			if (labelBestSize[lbl] < sz) {
				labelBestSize[lbl] = sz;
				labelBestSizeRect[lbl][0] = i;
				labelBestSizeRect[lbl][1] = j;
			}
		}
}

bool sameLabel (int lbl, int x, int y, int h, int w) {
	
	for (int i = x; i < x + h; ++i)
		for (int j = y; j < y + w; ++j)
			if (labels_need[i][j] != lbl )
				return false;
				
	return true;
}

void solve () {
	
	clock_t begin_t = clock();
	
	int clusters[MAX_LABELS][3];
	int count_current[MAX_LABELS];
	int k = 20;
	int operCnt = maxOperations;
	int operDone = 0;
	
	int dumpId = 0;
	char haha[100];
	
	while (maxOperations > 0) {
	
		clock_t end_t = clock();
		double time_elapsed = (double)(end_t - begin_t) / CLOCKS_PER_SEC;
		
		if (time_elapsed >= 8 || operDone * 2 >= operCnt) {
			
			MAXSIZE = 15;
		}
		else
		if (time_elapsed >= 12 || ((operDone * 3)/2) >= operCnt) {
			MAXSIZE = 7;
		}
		if (time_elapsed >= 14.3) {
			break;
		}
		
		for (int i = 0; i < H; ++i)
			for (int j = 0; j < W; ++j) {
				current_image[i][j] = startImage[i][j];
				covered[i][j] = 0;
			}
		
		gaussianSmooth(current_image);
			
		posterizeImage(current_image, labels_current, clusters, count_current, k);

		for (int i = 0; i < H; ++i)	
			for(int j = 0; j < W; ++j)
			 {
			
				labels_need[i][j] = -1;
				
				double bestij = -10000;
				//int bestme = -10000;
				
				for (int l = 0; l < k; ++l) 
				if (count_current[l] > 0) {
				
					rgb afterfix ((clusters[l][0] + current_image[i][j].r)/2, 
								  (clusters[l][1] + current_image[i][j].g)/2,
								  (clusters[l][2] + current_image[i][j].b)/2);
								  
					double gainij = calculate_gain(current_image[i][j], targetImage[i][j], afterfix);	
					if (bestij < gainij) {
						bestij = gainij;
						labels_need[i][j] = l;
					}
				}
			}
		
		compute_squares(msize, labels_current, k);
		
		for (int i = 0; i < k; ++i) {
			
			if (labelBestSize[i] > 0) {
				int x = labelBestSizeRect[i][0];
				int y = labelBestSizeRect[i][1];
				mark_covered(x, y, labelBestSize[i], labelBestSize[i]);
			}
		}
		
		opertry.clear();
		
		for (int j = 0; j < W; ++j)
			for (int i = 0; i < H; ++i) 
				if (!covered[i][j] && labels_need[i][j] >= 0 && labels_need[i][j] < k && labelBestSize[labels_need[i][j]] > 0) {
				
					int maxwh = labelBestSize[labels_need[i][j]];
					int h = 1, w = 1, lbl = labels_need[i][j];
					int ax = labelBestSizeRect[lbl][0];
					int ay = labelBestSizeRect[lbl][1];
					bool hadd = false;
					
					for (int add = 1; add < maxwh; ++add) {
						
						if (i + h < H && j + w < W && !check_covered(i + h, j, 1, w + 1) && !check_covered(i, j+w, h+1, 1)) {
							
							if (sameLabel(lbl,i+h,j,1,w+1) && sameLabel(lbl, i, j+w, h+1, 1))
								{ ++h; ++w; }
							else {
								break;
							}
						}
					}
					
					while (1) {
						if (i + h < H && h < maxwh && !check_covered(i + h, j, 1, w) && sameLabel(lbl, i+h, j, 1, w))
							{++h; hadd = true;}
						else break;
					}
					
					if (!hadd) {
						while (1) {
							if (j + w < w && j < maxwh && !check_covered(i, j + w, h, 1) && sameLabel(lbl, i, j+w, h, 1))
								{++w;}
							else break;
						}
					}

					mark_covered(i, j, h, w);
					double gainmoney = gainOper2(i,j,ax,ay,h,w);
					
					if (gainmoney > 0) {
						opertry.push_back(rect(i,j,h,w,(long long)gainmoney));
					}
				}
		
		sort (opertry.begin(), opertry.end(), cmpGainRect);
		
		int cntDone = 50;
		if (time_elapsed >= 13.5) cntDone = maxOperations;
		
		for (int i = 0; i <(int)opertry.size() && i < maxOperations && cntDone > 0; ++i) {
			
			int lbl = labels_need[opertry[i].x][opertry[i].y];
			int ax = labelBestSizeRect[lbl][0];
			int ay = labelBestSizeRect[lbl][1];
			
			/*if (check_covered(ax, ay, opertry[i].x, opertry[i].y)) {
				continue;
			}*/
			
			cntDone--;
			//printf ("label %d\n", lbl);
			//printf ("oper: %d %d from %d %d, size %d %d\n", opertry[i].x, opertry[i].y, ax, ay, opertry[i].h, opertry[i].w);
			//printf ("gain %lld\n", opertry[i].gainval);
			//printf ("color need %d %d %d, color get %d %d %d\n", clusters[lbl][0], clusters[lbl][1], clusters[lbl][2],
			//		clusters[labels_current[ax][ay]][0], clusters[labels_current[ax][ay]][0], clusters[labels_current[ax][ay]][2]);
			//printf ("start "); startImage[opertry[i].x][opertry[i].y].print(); 
			//printf ("target "); targetImage[opertry[i].x][opertry[i].y].print(); 
			
			update_start_image(opertry[i].x, opertry[i].y, ax, ay, opertry[i].h, opertry[i].w);
			ans_oper.push_back (oper(opertry[i].x, opertry[i].y, ax, ay, opertry[i].h, opertry[i].w));
			//printf ("%d %d %d %d %d %d\n", opertry[i].x, opertry[i].y, ax, ay, opertry[i].h, opertry[i].w);
			mark_covered(opertry[i].x, opertry[i].y, opertry[i].h, opertry[i].w);
			
			operDone++;
			maxOperations--;
		}
		
		//printf ("--------------%d\n", (int)ans_oper.size());
		//exit (0);
	}
	
	//exit(0);
}

int main() {

	freopen("transformation.in","r",stdin);
	freopen("transformation.out","w",stdout);
	
	srand (101);
	
	input();
	solve();
	
	int totalOperSize = 0;
	
	unsigned int i;
	for (i = 0; i < ans_oper.size(); ++i) {
		
		if (ans_oper[i].h * ans_oper[i].w + totalOperSize > 100*H*W)
			break;
			
		//printf ("%d %d %d %d %d %d\n", ans_oper[i].x, ans_oper[i].y, ans_oper[i].x2, ans_oper[i].y2, ans_oper[i].h, ans_oper[i].w);
		totalOperSize += ans_oper[i].h * ans_oper[i].w;
	}
	
	unsigned int n = i;
	
	printf ("%d\n", (int)n);
	for (i = 0; i < n; ++i) {
			
		printf ("%d %d %d %d %d %d\n", ans_oper[i].x, ans_oper[i].y, ans_oper[i].x2, ans_oper[i].y2, ans_oper[i].h, ans_oper[i].w);
	}
	
return 0;
}

